@@ -1,6 +1,5 @@
 # --
-# Kernel/GenericInterface/Operation/Ticket/Common.pm - Ticket common operation functions
-# Copyright (C) 2001-2014 OTRS AG, http://otrs.com/
+# Copyright (C) 2001-2016 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -716,6 +715,12 @@ checks if the given pending time is valid.
         },
     );
 
+    my $Success = $CommonObject->ValidatePendingTime(
+        PendingTime => {
+            Diff => 10080,
+        },
+    );
+
     returns
     $Success = 1            # or 0
 
@@ -728,6 +733,18 @@ sub ValidatePendingTime {
     return if !$Param{PendingTime};
     return if !IsHashRefWithData( $Param{PendingTime} );
 
+    # if only the Diff attribute is present, check if it's a valid number and return.
+    # Nothing else needs to be checked in that case.
+    if ( keys %{ $Param{PendingTime} } == 1 && defined $Param{PendingTime}->{Diff} ) {
+        return if $Param{PendingTime}->{Diff} !~ m{\A \d+ \z}msx;
+        return 1;
+    }
+    elsif ( defined $Param{PendingTime}->{Diff} ) {
+
+        # the use of Diff along with any other option is forbidden
+        return;
+    }
+
     # check that no time attribute is empty or negative
     for my $TimeAttribute ( sort keys %{ $Param{PendingTime} } ) {
         return if $Param{PendingTime}->{$TimeAttribute} eq '';
@@ -1113,7 +1130,7 @@ checks if the given dynamic field value is valid.
 
     my $Success = $CommonObject->ValidateDynamicFieldValue(
         Value => [                      # Only for fields that can handle multiple values like
-            'some value',               #   Miltiselect
+            'some value',               #   Multiselect
             'some other value',
         ],
     );
@@ -1128,7 +1145,11 @@ sub ValidateDynamicFieldValue {
 
     # check needed stuff
     return if !IsHashRefWithData( $Self->{DynamicFieldLookup} );
-    return if !IsStringWithData( $Param{Value} );
+
+    # possible structures are string and array, no data inside is needed
+    if ( !IsString( $Param{Value} ) && ref $Param{Value} ne 'ARRAY' ) {
+        return;
+    }
 
     # get dynamic field config
     my $DynamicFieldConfig = $Self->{DynamicFieldLookup}->{ $Param{Name} };
@@ -1209,15 +1230,23 @@ sub SetDynamicFieldValue {
     my ( $Self, %Param ) = @_;
 
     # check needed stuff
-    for my $Needed (qw(Value Name UserID)) {
-        if ( !IsStringWithData( $Param{$Needed} ) ) {
+    for my $Needed (qw(Name UserID)) {
+        if ( !IsString( $Param{$Needed} ) ) {
             return {
                 Success      => 0,
-                ErrorMessage => "SetDynamicFieldValue() Got no $Needed!"
+                ErrorMessage => "SetDynamicFieldValue() Invalid value for $Needed, just string is allowed!"
             };
         }
     }
 
+    # check value structure
+    if ( !IsString( $Param{Value} ) && ref $Param{Value} ne 'ARRAY' ) {
+        return {
+            Success      => 0,
+            ErrorMessage => "SetDynamicFieldValue() Invalid value for Value, just string and array are allowed!"
+        };
+    }
+
     return if !IsHashRefWithData( $Self->{DynamicFieldLookup} );
 
     # get dynamic field config
@@ -1302,6 +1331,17 @@ sub CreateAttachment {
 
 =item CheckCreatePermissions ()
 
+Tests if the user have the permissions to create a ticket on a determined queue
+
+    my $Result = $CommonObject->CheckCreatePermissions(
+        Ticket     => $TicketHashReference,
+        UserID     => 123,                      # or 'CustomerLogin'
+        UserType   => 'Agent',                  # or 'Customer'
+    );
+
+returns:
+    $Success = 1                                # if everything is OK
+
 =cut
 
 sub CheckCreatePermissions {
@@ -1318,10 +1358,9 @@ sub CheckCreatePermissions {
     my %UserGroups;
 
     if ( $Param{UserType} ne 'Customer' ) {
-        %UserGroups = $Kernel::OM->Get('Kernel::System::Group')->GroupMemberList(
+        %UserGroups = $Kernel::OM->Get('Kernel::System::Group')->PermissionUserGet(
             UserID => $Param{UserID},
             Type   => 'create',
-            Result => 'HASH',
         );
     }
     else {
@@ -1346,6 +1385,45 @@ sub CheckCreatePermissions {
     return 1;
 }
 
+=item CheckAccessPermissions()
+
+Tests if the user have access permissions over a ticket
+
+    my $Result = $CommonObject->CheckAccessPermissions(
+        TicketID   => 123,
+        UserID     => 123,                      # or 'CustomerLogin'
+        UserType   => 'Agent',                  # or 'Customer'
+    );
+
+returns:
+    $Success = 1                                # if everything is OK
+
+=cut
+
+sub CheckAccessPermissions {
+    my ( $Self, %Param ) = @_;
+
+    # check needed stuff
+    for my $Needed (qw(TicketID UserID UserType)) {
+        if ( !$Param{$Needed} ) {
+            return;
+        }
+    }
+
+    my $TicketPermissionFunction = 'TicketPermission';
+    if ( $Param{UserType} eq 'Customer' ) {
+        $TicketPermissionFunction = 'TicketCustomerPermission';
+    }
+
+    my $Access = $Kernel::OM->Get('Kernel::System::Ticket')->$TicketPermissionFunction(
+        Type     => 'ro',
+        TicketID => $Param{TicketID},
+        UserID   => $Param{UserID},
+    );
+
+    return $Access;
+}
+
 =begin Internal:
 
 =item _ValidateUser()
