@@ -1,5 +1,5 @@
 # --
-# Copyright (C) 2001-2015 OTRS AG, http://otrs.com/
+# Copyright (C) 2001-2016 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -356,7 +356,10 @@ sub _MaskNew {
 
     # generate CalendarOptionStrg
     my %CalendarList;
-    for my $CalendarNumber ( '', 1 .. 50 ) {
+
+    my $Maximum = $ConfigObject->Get("MaximumCalendarNumber") || 50;
+
+    for my $CalendarNumber ( '', 1 .. $Maximum ) {
         if ( $ConfigObject->Get("TimeVacationDays::Calendar$CalendarNumber") ) {
             $CalendarList{$CalendarNumber} = "Calendar $CalendarNumber - "
                 . $ConfigObject->Get( "TimeZone::Calendar" . $CalendarNumber . "Name" );
