@@ -1,9 +1,9 @@
 # --
-# Copyright (C) 2001-2020 OTRS AG, https://otrs.com/
+# Copyright (C) 2001-2016 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::System::Console::Command::Maint::Stats::Dashboard::Generate;
@@ -11,10 +11,9 @@ package Kernel::System::Console::Command::Maint::Stats::Dashboard::Generate;
 use strict;
 use warnings;
 
-use parent qw(Kernel::System::Console::BaseCommand);
+use base qw(Kernel::System::Console::BaseCommand);
 
 our @ObjectDependencies = (
-    'Kernel::Config',
     'Kernel::System::PID',
     'Kernel::System::User',
     'Kernel::System::JSON',
@@ -28,7 +27,7 @@ sub Configure {
     $Self->Description('Generate statistics widgets for the dashboard.');
     $Self->AddOption(
         Name        => 'number',
-        Description => "Statistic number as shown in the overview of AgentStats.",
+        Description => "Stats number (as shown on overview in AgentStats).",
         Required    => 0,
         HasValue    => 1,
         ValueRegex  => qr/\d+/smx,
@@ -79,8 +78,6 @@ sub Run {
         UserID => 1,
     );
 
-    my $DefaultLanguage = $Kernel::OM->Get('Kernel::Config')->Get('DefaultLanguage') || 'en';
-
     STATID:
     for my $StatID ( sort keys %{ $Stats || {} } ) {
 
@@ -92,7 +89,7 @@ sub Run {
         $Self->Print("<yellow>Stat $Stat{StatNumber}: $Stat{Title}</yellow>\n");
 
         # now find out all users which have this statistic enabled in their dashboard
-        my $DashboardActiveSetting   = 'UserDashboard' . ( 1000 + $StatID ) . "-Stats";
+        my $DashboardActiveSetting = 'UserDashboard' . ( 1000 + $StatID ) . "-Stats";
         my %UsersWithActivatedWidget = $Kernel::OM->Get('Kernel::System::User')->SearchPreferences(
             Key   => $DashboardActiveSetting,
             Value => 1,
@@ -105,8 +102,6 @@ sub Run {
         USERID:
         for my $UserID ( sort keys %UsersWithActivatedWidget ) {
 
-            my $StartTime = time();    ## no critic
-
             # ignore invalid users
             my %UserData = $Kernel::OM->Get('Kernel::System::User')->GetUserData(
                 UserID        => $UserID,
@@ -130,31 +125,13 @@ sub Run {
                 print STDERR $Kernel::OM->Get('Kernel::System::Main')->Dump($UserGetParam);
             }
 
-            $Kernel::OM->ObjectsDiscard(
-                Objects => ['Kernel::Language'],
-            );
-
-            $Kernel::OM->ObjectParamAdd(
-                'Kernel::Language' => {
-                    UserLanguage => $UserData{UserLanguage} || $DefaultLanguage,
-                },
-            );
-
-            # Now run the stat to fill the cache with the current parameters (passing the
-            #   user language for the correct caching).
+            # now run the stat to fill the cache with the current parameters
             my $Result = $Kernel::OM->Get('Kernel::System::Stats')->StatsResultCacheCompute(
                 StatID       => $StatID,
-                UserGetParam => {
-                    %{$UserGetParam},
-                    UserLanguage => $UserData{UserLanguage} || $DefaultLanguage,
-                },
-                UserID => $UserID
+                UserGetParam => $UserGetParam,
+                UserID       => $UserID
             );
 
-            if ( $Self->GetOption('debug') ) {
-                print STDERR sprintf( "DEBUG: time taken: %ss\n", time() - $StartTime );    ## no critic
-            }
-
             if ( !$Result ) {
                 $Self->PrintError("        Stat calculation was not successful.");
             }
@@ -172,3 +149,15 @@ sub PostRun {
 }
 
 1;
+
+=back
+
+=head1 TERMS AND CONDITIONS
+
+This software is part of the OTRS project (L<http://otrs.org/>).
+
+This software comes with ABSOLUTELY NO WARRANTY. For details, see
+the enclosed file COPYING for license information (AGPL). If you
+did not receive this file, see L<http://www.gnu.org/licenses/agpl.txt>.
+
+=cut
