@@ -14,10 +14,10 @@ use warnings;
 use utf8;
 
 use MIME::Base64;
+use HTML::Truncate;
 
 our @ObjectDependencies = (
     'Kernel::Config',
-    'Kernel::System::Encode',
     'Kernel::System::Log',
 );
 
@@ -80,9 +80,6 @@ sub ToAscii {
         }
     }
 
-    # turn on utf8 flag (bug#10970, bug#11596 and bug#12097)
-    $Kernel::OM->Get('Kernel::System::Encode')->EncodeInput( \$Param{String} );
-
     # get length of line for forcing line breakes
     my $LineLength = $Self->{'Ticket::Frontend::TextAreaNote'} || 78;
 
@@ -485,14 +482,9 @@ sub ToAscii {
     }
     {
         my $Chr = chr( $2 );
-
-        # Make sure we get valid UTF8 code points, but skip characters from 128 to 255
-        #   (inclusive), since they are by default internally not encoded as UTF-8 for
-        #   backward compatibility reasons. See bug#12457 for more information.
-        if ( $2 < 128 || $2 > 255 ) {
-            Encode::_utf8_off($Chr);
-            $Chr = Encode::decode('utf-8', $Chr, 0);
-        }
+        # make sure we get valid UTF8 code points
+        Encode::_utf8_off( $Chr);
+        $Chr = Encode::decode('utf-8', $Chr, 0);
 
         if ( $Chr ) {
             $Chr;
@@ -508,18 +500,12 @@ sub ToAscii {
     }
     {
         my $ChrOrig = $1;
-        my $Dec = hex( $2 );
-        if ( $Dec ) {
-            my $Chr = chr( $Dec );
-
-            # Make sure we get valid UTF8 code points, but skip characters from 128 to 255
-            #   (inclusive), since they are by default internally not encoded as UTF-8 for
-            #   backward compatibility reasons. See bug#12457 for more information.
-            if ( $Dec < 128 || $Dec > 255 ) {
-                Encode::_utf8_off($Chr);
-                $Chr = Encode::decode('utf-8', $Chr, 0);
-            }
-
+        my $Hex = hex( $2 );
+        if ( $Hex ) {
+            my $Chr = chr( $Hex );
+            # make sure we get valid UTF8 code points
+            Encode::_utf8_off( $Chr);
+            $Chr = Encode::decode('utf-8', $Chr, 0);
             if ( $Chr ) {
                 $Chr;
             }
@@ -788,7 +774,7 @@ sub LinkQuote {
 
         # add target to existing "<a href"
         ${$String} =~ s{
-            (<a\s{1,10})([^>]+)>
+            (<a\s{1,10})(.+?)>
         }
         {
             my $Start = $1;
@@ -802,30 +788,28 @@ sub LinkQuote {
         }egxsi;
     }
 
-    my $Marker = "§" x 10;
-
-    # Remove existing <a>...</a> tags and their content to be re-inserted later, this must not be quoted.
-    # Also remove other tags to avoid quoting in tag parameters.
+    # remove existing "<a href" on all other tags (to find not linked urls) and remember it
     my $Counter = 0;
-    my %TagHash;
+    my %LinkHash;
     ${$String} =~ s{
-        (<a\s[^>]*?>[^>]*</a>|<[^>]+?>)
+        (<a\s.+?>.+?</a>)
     }
     {
         my $Content = $1;
-        my $Key     = "${Marker}TagHash-$Counter${Marker}";
-        $TagHash{$Counter++} = $Content;
+        $Counter++;
+        my $Key  = "############LinkHash-$Counter############";
+        $LinkHash{$Key} = $Content;
         $Key;
     }egxism;
 
-    # Add <a> tags for URLs in the content.
+    # replace not "<a href" found urls and link it
     my $Target = '';
     if ( $Param{Target} ) {
         $Target = " target=\"$Param{Target}\"";
     }
     ${$String} =~ s{
         (                                          # $1 greater-than and less-than sign
-            > | < | \s+ | §{10} |
+            > | < | \s+ | \#{6} |
             (?: &[a-zA-Z0-9]+; )                   # get html entities
         )
         (                                          # $2
@@ -862,7 +846,7 @@ sub LinkQuote {
                 | (?: &[a-zA-Z0-9]+; )+            # html entities
                 | $                                # bug# 2715
             )
-            | §{10}                                # ending TagHash
+            | \#{6}                                # ending LinkHash
         )
     }
     {
@@ -898,8 +882,12 @@ sub LinkQuote {
         $Start . "<a href=\"$HrefLink\"$Target title=\"$HrefLink\">$DisplayLink<\/a>" . $End;
     }egxism;
 
-    # Re-add previously removed tags.
-    ${$String} =~ s{${Marker}TagHash-(\d+)${Marker}}{$TagHash{$1}}egsxim;
+    my ( $Key, $Value );
+
+    # add already existing "<a href" again
+    while ( ( $Key, $Value ) = each(%LinkHash) ) {
+        ${$String} =~ s{$Key}{$Value};
+    }
 
     # check ref && return result like called
     if ($StringScalar) {
@@ -914,14 +902,16 @@ To remove/strip active html tags/addons (javascript, applets, embeds and objects
 from html strings.
 
     my %Safe = $HTMLUtilsObject->Safety(
-        String       => $HTMLString,
-        NoApplet     => 1,
-        NoObject     => 1,
-        NoEmbed      => 1,
-        NoSVG        => 1,
-        NoIntSrcLoad => 0,
-        NoExtSrcLoad => 1,
-        NoJavaScript => 1,
+        String         => $HTMLString,
+        NoApplet       => 1,
+        NoObject       => 1,
+        NoEmbed        => 1,
+        NoSVG          => 1,
+        NoImg          => 1,
+        NoIntSrcLoad   => 0,
+        NoExtSrcLoad   => 1,
+        NoJavaScript   => 1,
+        ReplacementStr => 'string',          # optional, string to show instead of applet, object, embed, svg and img tags
     );
 
 also string ref is possible
@@ -932,6 +922,7 @@ also string ref is possible
         NoObject     => 1,
         NoEmbed      => 1,
         NoSVG        => 1,
+        NoImg        => 1,
         NoIntSrcLoad => 0,
         NoExtSrcLoad => 1,
         NoJavaScript => 1,
@@ -1041,18 +1032,20 @@ sub Safety {
             }egsxim;
         }
 
-        # remove HTTP refirects
+        # remove HTTP redirects
         $Replaced += ${$String} =~ s{
             $TagStart meta [^>]+? http-equiv=('|"|)refresh [^>]+? $TagEnd
         }
         {}sgxim;
 
+        my $ReplacementStr = $Param{ReplacementStr} // '';
+
         # remove <applet> tags
         if ( $Param{NoApplet} ) {
             $Replaced += ${$String} =~ s{
                 $TagStart applet.*? $TagEnd (.*?) $TagStart /applet \s* $TagEnd
             }
-            {}sgxim;
+            {$ReplacementStr}sgxim;
         }
 
         # remove <Object> tags
@@ -1060,7 +1053,7 @@ sub Safety {
             $Replaced += ${$String} =~ s{
                 $TagStart object.*? $TagEnd (.*?) $TagStart /object \s* $TagEnd
             }
-            {}sgxim;
+            {$ReplacementStr}sgxim;
         }
 
         # remove <svg> tags
@@ -1068,7 +1061,15 @@ sub Safety {
             $Replaced += ${$String} =~ s{
                 $TagStart svg.*? $TagEnd (.*?) $TagStart /svg \s* $TagEnd
             }
-            {}sgxim;
+            {$ReplacementStr}sgxim;
+        }
+
+        # remove <img> tags
+        if ( $Param{NoImg} ) {
+            $Replaced += ${$String} =~ s{
+                $TagStart img.*? (.*?) \s* $TagEnd
+            }
+            {$ReplacementStr}sgxim;
         }
 
         # remove <embed> tags
@@ -1076,7 +1077,7 @@ sub Safety {
             $Replaced += ${$String} =~ s{
                 $TagStart embed.*? $TagEnd
             }
-            {}sgxim;
+            {$ReplacementStr}sgxim;
         }
 
         # check each html tag
@@ -1093,12 +1094,6 @@ sub Safety {
                 }
                 {$2}sgxim;
 
-                # remove entities in tag
-                $Replaced += $Tag =~ s{
-                    (&[{].+?[}])
-                }
-                {}sgxim;
-
                 # remove javascript in a href links or src links
                 $Replaced += $Tag =~ s{
                     ((?:\s|;|/)(?:background|url|src|href)=)
@@ -1189,7 +1184,7 @@ sub EmbeddedImagesExtract {
     if ( ref $Param{AttachmentsRef} ne 'ARRAY' ) {
         $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
-            Message  => "Need DocumentRef!"
+            Message  => "Need AttachmentsRef!"
         );
         return;
     }
@@ -1220,6 +1215,101 @@ sub EmbeddedImagesExtract {
     return 1;
 }
 
+=item HTMLTruncate()
+
+DEPRECATED: This function will be removed in further versions of OTRS
+
+truncate an HTML string to certain amount of characters without loosing the HTML tags, the resulting
+string will contain the specified amount of text characters plus the HTML tags, and ellipsis string.
+
+special characters like &aacute; in HTML code are considered as just one character.
+
+    my $HTML = $HTMLUtilsObject->HTMLTruncate(
+        String   => $String,
+        Chars    => 123,
+        Ellipsis => '...',              # optional (defaults to HTML &#8230;) string to indicate
+                                        #    that the HTML was truncated until that point
+        UTF8Mode => 0,                  # optional 1 or 0 (defaults to 0)
+        OnSpace  => 0,                  # optional 1 or 0 (defaults to 0) if enabled, prevents to
+                                        #    truncate in a middle of a word, but in the space before
+    );
+
+returns
+
+    $HTML => 'some HTML code'           # or false in case of a failure
+
+=cut
+
+sub HTMLTruncate {
+    my ( $Self, %Param ) = @_;
+
+    # check needed
+    for my $Needed (qw(String Chars)) {
+
+        if ( !$Param{$Needed} ) {
+            $Kernel::OM->Get('Kernel::System::Log')->Log(
+                Priority => 'error',
+                Message  => "Need $Needed!",
+            );
+            return;
+        }
+    }
+
+    # translate params for compatibility reasons with HTML::Truncate
+    my %CompatibilityParams = (
+        'utf8_mode' => $Param{UTF8Mode} ? 1 : 0,
+        'on_space'  => $Param{OnSpace}  ? 1 : 0,
+        'chars'     => $Param{Chars},
+        'repair'    => 1,
+    );
+
+    if ( defined $Param{Ellipsis} ) {
+        $CompatibilityParams{ellipsis} = $Param{Ellipsis};
+    }
+
+    # create new HTML truncate object (with the specified options)
+    my $HTMLTruncateObject;
+    eval {
+        $HTMLTruncateObject = HTML::Truncate->new(%CompatibilityParams);
+    };
+
+    if ( !$HTMLTruncateObject ) {
+        $Kernel::OM->Get('Kernel::System::Log')->Log(
+            Priority => 'error',
+            Message  => "Could not create HTMLTruncateObject: $@",
+        );
+        return;
+    }
+
+    # sanitize the string
+    my %Safe = $Self->Safety(
+        String         => $Param{String},
+        NoApplet       => 1,
+        NoObject       => 1,
+        NoEmbed        => 1,
+        NoSVG          => 1,
+        NoImg          => 1,
+        NoIntSrcLoad   => 1,
+        NoExtSrcLoad   => 1,
+        NoJavaScript   => 1,
+        ReplacementStr => '✂︎',
+    );
+
+    # truncate the HTML input string
+    my $Result;
+    if ( !eval { $Result = $HTMLTruncateObject->truncate( $Safe{String} ) } ) {
+
+        $Kernel::OM->Get('Kernel::System::Log')->Log(
+            Priority => 'error',
+            Message  => 'Truncating string failed: ' . $@,
+        );
+
+        return;
+    }
+
+    return $Result;
+}
+
 1;
 
 =back
