@@ -1,5 +1,5 @@
 # --
-# Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
+# Copyright (C) 2001-2016 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -22,9 +22,9 @@ our @ObjectDependencies = (
     'Kernel::System::CustomerUser',
     'Kernel::System::DynamicField',
     'Kernel::System::DynamicField::Backend',
+    'Kernel::System::Encode',
     'Kernel::System::HTMLUtils',
     'Kernel::System::Log',
-    'Kernel::System::Notification',
     'Kernel::System::Queue',
     'Kernel::System::Salutation',
     'Kernel::System::Signature',
@@ -32,6 +32,9 @@ our @ObjectDependencies = (
     'Kernel::System::SystemAddress',
     'Kernel::System::Ticket',
     'Kernel::System::User',
+    'Kernel::Output::HTML::Layout',
+    'Kernel::System::JSON',
+
 );
 
 =head1 NAME
@@ -687,7 +690,7 @@ sub AutoResponse {
         );
 
         $AutoResponse{Text} = $Kernel::OM->Get('Kernel::System::HTMLUtils')->DocumentComplete(
-            Charset => $AutoResponse{Charset},
+            Charset => 'utf-8',
             String  => $AutoResponse{Text},
         );
     }
@@ -695,52 +698,50 @@ sub AutoResponse {
     return %AutoResponse;
 }
 
-=item NotificationAgent()
+=item NotificationEvent()
 
-generate response
+replace all OTRS smart tags in the notification body and subject
 
-NotificationAgent
-    TicketID
-        Owner
-        Responsible
-        CUSTOMER_DATA
-    ArticleID
-        CUSTOMER_SUBJECT
-        CUSTOMER_EMAIL
-    UserID
-
-    To
-    Cc
-    Bcc
-    Subject
-    Body
-    ContentType
-    SenderAddress
-    SenderRealname
-
-    my %NotificationAgent = $TemplateGeneratorObject->NotificationAgent(
-        Type        => 'Move', # notification types, see database
-        TicketID    => 123,
-        RecipientID => 123,
-        UserID      => 123,
+    my %NotificationEvent = $TemplateGeneratorObject->NotificationEvent(
+        TicketID              => 123,
+        Recipient             => $UserDataHashRef,          # Agent or Customer data get result
+        Notification          => $NotificationDataHashRef,
+        CustomerMessageParams => $ArticleHashRef,           # optional
+        UserID                => 123,
     );
 
 =cut
 
-sub NotificationAgent {
+sub NotificationEvent {
     my ( $Self, %Param ) = @_;
 
     # check needed stuff
-    for (qw(TicketID Type RecipientID UserID)) {
-        if ( !$Param{$_} ) {
+    for my $Needed (qw(TicketID Notification Recipient UserID)) {
+        if ( !$Param{$Needed} ) {
             $Kernel::OM->Get('Kernel::System::Log')->Log(
                 Priority => 'error',
-                Message  => "Need $_!"
+                Message  => "Need $Needed!",
             );
             return;
         }
     }
 
+    if ( !IsHashRefWithData( $Param{Notification} ) ) {
+        $Kernel::OM->Get('Kernel::System::Log')->Log(
+            Priority => 'error',
+            Message  => "Notification is invalid!",
+        );
+        return;
+    }
+
+    my %Notification = %{ $Param{Notification} };
+
+    # exchanging original reference prevent it to grow up
+    if ( ref $Param{CustomerMessageParams} && ref $Param{CustomerMessageParams} eq 'HASH' ) {
+        my %LocalCustomerMessageParams = %{ $Param{CustomerMessageParams} };
+        $Param{CustomerMessageParams} = \%LocalCustomerMessageParams;
+    }
+
     # get ticket object
     my $TicketObject = $Kernel::OM->Get('Kernel::System::Ticket');
 
@@ -750,17 +751,80 @@ sub NotificationAgent {
         DynamicFields => 0,
     );
 
-    # get old article for quoting
+    # get last article from customer
     my %Article = $TicketObject->ArticleLastCustomerArticle(
         TicketID      => $Param{TicketID},
         DynamicFields => 0,
     );
 
-    for (qw(From To Cc Subject Body)) {
-        if ( !$Param{CustomerMessageParams}->{$_} ) {
-            $Param{CustomerMessageParams}->{$_} = $Article{$_} || '';
+    # get last article from agent
+    my @ArticleBoxAgent = $TicketObject->ArticleGet(
+        TicketID      => $Param{TicketID},
+        UserID        => $Param{UserID},
+        DynamicFields => 0,
+    );
+
+    my %ArticleAgent;
+
+    ARTICLE:
+    for my $Article ( reverse @ArticleBoxAgent ) {
+
+        next ARTICLE if $Article->{SenderType} ne 'agent';
+
+        %ArticleAgent = %{$Article};
+
+        last ARTICLE;
+    }
+
+    # get  HTMLUtils object
+    my $HTMLUtilsObject = $Kernel::OM->Get('Kernel::System::HTMLUtils');
+
+    # set the accounted time as part of the articles information
+    ARTICLE:
+    for my $ArticleData ( \%Article, \%ArticleAgent ) {
+
+        next ARTICLE if !$ArticleData->{ArticleID};
+
+        # get accounted time
+        my $AccountedTime = $TicketObject->ArticleAccountedTimeGet(
+            ArticleID => $ArticleData->{ArticleID},
+        );
+
+        $ArticleData->{TimeUnit} = $AccountedTime;
+    }
+
+    # get system default language
+    my $DefaultLanguage = $Kernel::OM->Get('Kernel::Config')->Get('DefaultLanguage') || 'en';
+
+    my $Languages = [ $Param{Recipient}->{UserLanguage}, $DefaultLanguage, 'en' ];
+
+    my $Language;
+    LANGUAGE:
+    for my $Item ( @{$Languages} ) {
+        next LANGUAGE if !$Item;
+        next LANGUAGE if !$Notification{Message}->{$Item};
+
+        # set language
+        $Language = $Item;
+        last LANGUAGE;
+    }
+
+    # if no language, then take the first one available
+    if ( !$Language ) {
+        my @NotificationLanguages = sort keys %{ $Notification{Message} };
+        $Language = $NotificationLanguages[0];
+    }
+
+    # copy the correct language message attributes to a flat structure
+    for my $Attribute (qw(Subject Body ContentType)) {
+        $Notification{$Attribute} = $Notification{Message}->{$Language}->{$Attribute};
+    }
+
+    for my $Key (qw(From To Cc Subject Body ContentType ArticleType)) {
+        if ( !$Param{CustomerMessageParams}->{$Key} ) {
+            $Param{CustomerMessageParams}->{$Key} = $Article{$Key} || '';
         }
-        chomp $Param{CustomerMessageParams}->{$_};
+        chomp $Param{CustomerMessageParams}->{$Key};
     }
 
     # format body (only if longer the 86 chars)
@@ -784,25 +848,29 @@ sub NotificationAgent {
     }
 
     # fill up required attributes
-    for (qw(Subject Body)) {
-        if ( !$Param{CustomerMessageParams}->{$_} ) {
-            $Param{CustomerMessageParams}->{$_} = "No $_";
+    for my $Text (qw(Subject Body)) {
+        if ( !$Param{CustomerMessageParams}->{$Text} ) {
+            $Param{CustomerMessageParams}->{$Text} = "No $Text";
         }
     }
 
-    # get recipient
-    my %User = $Kernel::OM->Get('Kernel::System::User')->GetUserData(
-        UserID => $Param{RecipientID},
-        Valid  => 1,
-    );
+    my $Start = '<';
+    my $End   = '>';
+    if ( $Notification{ContentType} =~ m{text\/html} ) {
+        $Start = '&lt;';
+        $End   = '&gt;';
+    }
 
-    # get user language
-    my $Language = $User{UserLanguage} || $Kernel::OM->Get('Kernel::Config')->Get('DefaultLanguage') || 'en';
+    # replace <OTRS_CUSTOMER_DATA_*> tags early from CustomerMessageParams, the rests will be replaced
+    # by ticket customer user
+    KEY:
+    for my $Key ( sort keys %{ $Param{CustomerMessageParams} || {} } ) {
 
-    # get notification data
-    my %Notification = $Kernel::OM->Get('Kernel::System::Notification')->NotificationGet(
-        Name => $Language . '::Agent::' . $Param{Type},
-    );
+        next KEY if !$Param{CustomerMessageParams}->{$Key};
+
+        $Notification{Body} =~ s/${Start}OTRS_CUSTOMER_DATA_$Key${End}/$Param{CustomerMessageParams}->{$Key}/gi;
+        $Notification{Subject} =~ s/<OTRS_CUSTOMER_DATA_$Key>/$Param{CustomerMessageParams}->{$Key}{$_}/gi;
+    }
 
     # do text/plain to text/html convert
     if ( $Self->{RichText} && $Notification{ContentType} =~ /text\/plain/i ) {
@@ -821,166 +889,51 @@ sub NotificationAgent {
     }
 
     # get notify texts
-    for (qw(Subject Body)) {
-        if ( !$Notification{$_} ) {
-            $Notification{$_} = "No Notification $_ for $Param{Type} found!";
+    for my $Text (qw(Subject Body)) {
+        if ( !$Notification{$Text} ) {
+            $Notification{$Text} = "No Notification $Text for $Param{Type} found!";
         }
     }
 
     # replace place holder stuff
     $Notification{Body} = $Self->_Replace(
-        RichText    => $Self->{RichText},
-        Text        => $Notification{Body},
-        RecipientID => $Param{RecipientID},
-        Data        => $Param{CustomerMessageParams},
-        TicketID    => $Param{TicketID},
-        UserID      => $Param{UserID},
-        Language    => $Language,
+        RichText  => $Self->{RichText},
+        Text      => $Notification{Body},
+        Recipient => $Param{Recipient},
+        Data      => $Param{CustomerMessageParams},
+        DataAgent => \%ArticleAgent,
+        TicketID  => $Param{TicketID},
+        UserID    => $Param{UserID},
+        Language  => $Language,
     );
     $Notification{Subject} = $Self->_Replace(
-        RichText    => 0,
-        Text        => $Notification{Subject},
-        RecipientID => $Param{RecipientID},
-        Data        => $Param{CustomerMessageParams},
-        TicketID    => $Param{TicketID},
-        UserID      => $Param{UserID},
-        Language    => $Language,
+        RichText  => 0,
+        Text      => $Notification{Subject},
+        Recipient => $Param{Recipient},
+        Data      => $Param{CustomerMessageParams},
+        DataAgent => \%ArticleAgent,
+        TicketID  => $Param{TicketID},
+        UserID    => $Param{UserID},
+        Language  => $Language,
     );
 
-    # prepare subject (insert old subject)
-    $Param{CustomerMessageParams}->{Subject} = $TicketObject->TicketSubjectClean(
-        TicketNumber => $Ticket{TicketNumber},
-        Subject      => $Param{CustomerMessageParams}->{Subject} || '',
-    );
-    if ( $Notification{Subject} =~ /<OTRS_CUSTOMER_SUBJECT\[(.+?)\]>/ ) {
-        my $SubjectChar = $1;
-        $Param{CustomerMessageParams}->{Subject} =~ s/^(.{$SubjectChar}).*$/$1 [...]/;
-
-        $Notification{Subject}
-            =~ s/<OTRS_CUSTOMER_SUBJECT\[.+?\]>/$Param{CustomerMessageParams}->{Subject}/g;
-    }
     $Notification{Subject} = $TicketObject->TicketSubjectBuild(
         TicketNumber => $Ticket{TicketNumber},
         Subject      => $Notification{Subject} || '',
         Type         => 'New',
     );
 
-    # add urls and verify to be full html document
+    # add URLs and verify to be full HTML document
     if ( $Self->{RichText} ) {
 
         $Notification{Body} = $Kernel::OM->Get('Kernel::System::HTMLUtils')->LinkQuote(
             String => $Notification{Body},
         );
-
-        $Notification{Body} = $Kernel::OM->Get('Kernel::System::HTMLUtils')->DocumentComplete(
-            Charset => $Notification{Charset},
-            String  => $Notification{Body},
-        );
     }
 
     return %Notification;
 }
 
-=item NotificationCustomer()
-
-generate response
-
-NotificationCustomer
-    TicketID
-        Owner
-        Responsible
-        CUSTOMER_DATA
-    ArticleID
-        CUSTOMER_SUBJECT
-        CUSTOMER_EMAIL
-    UserID
-
-    To
-    Cc
-    Bcc
-    Subject
-    Body
-    ContentType
-    SenderAddress
-    SenderRealname
-
-    my %NotificationCustomer = $TemplateGeneratorObject->NotificationCustomer(
-        Type       => 'Move', # notification types, see database
-        TicketID   => 123,
-        ArticleID  => 123,
-        NotificationCustomerID => 123
-        UserID     => 123,
-    );
-
-=cut
-
-sub NotificationCustomer {
-    my ( $Self, %Param ) = @_;
-
-    # check needed stuff
-    for (qw(TicketID Type UserID)) {
-        if ( !$Param{$_} ) {
-            $Kernel::OM->Get('Kernel::System::Log')->Log(
-                Priority => 'error',
-                Message  => "Need $_!"
-            );
-            return;
-        }
-    }
-
-    my %Ticket = $Kernel::OM->Get('Kernel::System::Ticket')->TicketGet(
-        TicketID      => $Param{TicketID},
-        DynamicFields => 0,
-    );
-
-    # get queue object
-    my $QueueObject = $Kernel::OM->Get('Kernel::System::Queue');
-
-    my %Queue;
-    if ( $Param{QueueID} ) {
-        %Queue = $QueueObject->QueueGet( ID => $Param{QueueID} );
-    }
-
-    my %User = $Kernel::OM->Get('Kernel::System::CustomerUser')->GetPreferences(
-        UserID => $Ticket{CustomerUserID},
-    );
-
-    # get user language
-    my $Language = $User{UserLanguage} || $Kernel::OM->Get('Kernel::Config')->Get('DefaultLanguage') || 'en';
-
-    # get notification data
-    my %Notification = $Kernel::OM->Get('Kernel::System::Notification')->NotificationGet(
-        Name => $Language . '::Customer::' . $Param{Type},
-    );
-
-    # get notify texts
-    for (qw(Subject Body)) {
-        if ( !$Notification{$_} ) {
-            $Notification{$_} = "No Notification $_ for $Param{Type} found!";
-        }
-    }
-
-    $Notification{ContentType} = 'text/plain';
-
-    # replace place holder stuff
-    for (qw(Subject Body)) {
-        $Notification{$_} = $Self->_Replace(
-            RichText => 0,
-            Text     => $Notification{$_},
-            TicketID => $Param{TicketID},
-            UserID   => $Param{UserID},
-            Language => $Language,
-        );
-    }
-
-    # get sender attributes
-    my %Address = $QueueObject->GetSystemAddress( QueueID => $Ticket{QueueID} );
-    $Notification{SenderAddress}  = $Address{Email};
-    $Notification{SenderRealname} = $Address{RealName};
-
-    return %Notification;
-}
-
 =begin Internal:
 
 =cut
@@ -1067,38 +1020,73 @@ sub _Replace {
         );
     }
 
+    # get config object
     my $ConfigObject = $Kernel::OM->Get('Kernel::Config');
 
-    # Replace config options.
+    # special replace from secret config options
+    my @SecretConfigOptions = qw(
+        DatabasePw
+        SearchUserPw
+        UserPw
+        SendmailModule::AuthPassword
+        AuthModule::Radius::Password
+        PGP::Key::Password
+        Customer::AuthModule::DB::CustomerPassword
+        Customer::AuthModule::Radius::Password
+        PublicFrontend::AuthPassword
+    );
+
+    # replace the secret config options before the normal config options
+    for my $SecretConfigOption (@SecretConfigOptions) {
+
+        my $Tag = $Start . 'OTRS_CONFIG_' . $SecretConfigOption . $End;
+        $Param{Text} =~ s{$Tag}{xxx}gx;
+    }
+
+    # replace config options
     my $Tag = $Start . 'OTRS_CONFIG_';
-    $Param{Text} =~ s{$Tag(.+?)$End}{
-        my $Replace = '';
-        # Mask secret config options.
-        if ($1 =~ m{(Password|Pw)\d*$}smxi) {
-            $Replace = 'xxx';
-        }
-        else {
-            $Replace = $ConfigObject->Get($1) // '';
-        }
-        $Replace;
-    }egx;
+    $Param{Text} =~ s{$Tag(.+?)$End}{$ConfigObject->Get($1) // ''}egx;
 
     # cleanup
     $Param{Text} =~ s/$Tag.+?$End/-/gi;
 
+    my %Recipient = %{ $Param{Recipient} || {} };
+
     # get user object
     my $UserObject = $Kernel::OM->Get('Kernel::System::User');
 
-    # get recipient data and replace it with <OTRS_...
-    $Tag = $Start . 'OTRS_';
-    if ( $Param{RecipientID} ) {
+    if ( !%Recipient && $Param{RecipientID} ) {
 
-        my %Recipient = $UserObject->GetUserData(
+        %Recipient = $UserObject->GetUserData(
             UserID        => $Param{RecipientID},
             NoOutOfOffice => 1,
         );
+    }
 
-        # html quoting of content
+    my $HashGlobalReplace = sub {
+        my ( $Tag, %H ) = @_;
+
+        # Generate one single matching string for all keys to save performance.
+        my $Keys = join '|', map {quotemeta} grep { defined $H{$_} } keys %H;
+
+        # Add all keys also as lowercase to be able to match case insensitive,
+        #   e. g. <OTRS_CUSTOMER_From> and <OTRS_CUSTOMER_FROM>.
+        for my $Key ( sort keys %H ) {
+            $H{ lc $Key } = $H{$Key};
+        }
+
+        $Param{Text} =~ s/(?:$Tag)($Keys)$End/$H{ lc $1 }/ieg;
+    };
+
+    # get recipient data and replace it with <OTRS_...
+    $Tag = $Start . 'OTRS_';
+
+    # include more readable tag <OTRS_NOTIFICATION_RECIPIENT
+    my $RecipientTag = $Start . 'OTRS_NOTIFICATION_RECIPIENT_';
+
+    if (%Recipient) {
+
+        # HTML quoting of content
         if ( $Param{RichText} ) {
             ATTRIBUTE:
             for my $Attribute ( sort keys %Recipient ) {
@@ -1109,17 +1097,18 @@ sub _Replace {
             }
         }
 
-        # replace it
-        ATTRIBUTE:
-        for my $Attribute ( sort keys %Recipient ) {
-            next ATTRIBUTE if !defined $Recipient{$Attribute};
-            $Param{Text} =~ s/$Tag$Attribute$End/$Recipient{$Attribute}/gi;
-        }
+        $HashGlobalReplace->( "$Tag|$RecipientTag", %Recipient );
     }
 
+    # cleanup
+    $Param{Text} =~ s/$RecipientTag.+?$End/-/gi;
+
     # get owner data and replace it with <OTRS_OWNER_...
     $Tag = $Start . 'OTRS_OWNER_';
 
+    # include more readable version <OTRS_TICKET_OWNER
+    my $OwnerTag = $Start . 'OTRS_TICKET_OWNER_';
+
     if ( $Ticket{OwnerID} ) {
 
         my %Owner = $UserObject->GetUserData(
@@ -1139,41 +1128,26 @@ sub _Replace {
             }
         }
 
-        # replace it
-        ATTRIBUTE:
-        for my $Attribute ( sort keys %Owner ) {
-            next ATTRIBUTE if !defined $Owner{$Attribute};
-            $Param{Text} =~ s/$Tag$Attribute$End/$Owner{$Attribute}/gi;
-        }
+        $HashGlobalReplace->( "$Tag|$OwnerTag", %Owner );
     }
 
     # cleanup
     $Param{Text} =~ s/$Tag.+?$End/-/gi;
-
-    my $HashGlobalReplace = sub {
-        my ( $Tag, %H ) = @_;
-
-        # Generate one single matching string for all keys to save performance.
-        my $Keys = join '|', map {quotemeta} grep { defined $H{$_} } keys %H;
-
-        # Add all keys also as lowercase to be able to match case insensitive,
-        #   e. g. <OTRS_CUSTOMER_From> and <OTRS_CUSTOMER_FROM>.
-        for my $Key ( sort keys %H ) {
-            $H{ lc $Key } = $H{$Key};
-        }
-
-        $Param{Text} =~ s/(?:$Tag)($Keys)$End/$H{ lc $1 }/ieg;
-    };
+    $Param{Text} =~ s/$OwnerTag.+?$End/-/gi;
 
     # get owner data and replace it with <OTRS_RESPONSIBLE_...
     $Tag = $Start . 'OTRS_RESPONSIBLE_';
+
+    # include more readable version <OTRS_TICKET_RESPONSIBLE
+    my $ResponsibleTag = $Start . 'OTRS_TICKET_RESPONSIBLE_';
+
     if ( $Ticket{ResponsibleID} ) {
         my %Responsible = $UserObject->GetUserData(
             UserID        => $Ticket{ResponsibleID},
             NoOutOfOffice => 1,
         );
 
-        # html quoting of content
+        # HTML quoting of content
         if ( $Param{RichText} ) {
 
             ATTRIBUTE:
@@ -1185,12 +1159,12 @@ sub _Replace {
             }
         }
 
-        # replace it
-        $HashGlobalReplace->( $Tag, %Responsible );
+        $HashGlobalReplace->( "$Tag|$ResponsibleTag", %Responsible );
     }
 
     # cleanup
     $Param{Text} =~ s/$Tag.+?$End/-/gi;
+    $Param{Text} =~ s/$ResponsibleTag.+?$End/-/gi;
 
     $Tag = $Start . 'OTRS_Agent_';
     my $Tag2        = $Start . 'OTRS_CURRENT_';
@@ -1199,7 +1173,7 @@ sub _Replace {
         NoOutOfOffice => 1,
     );
 
-    # html quoting of content
+    # HTML quoting of content
     if ( $Param{RichText} ) {
 
         ATTRIBUTE:
@@ -1218,7 +1192,6 @@ sub _Replace {
     $Param{Text} =~ s/$Start OTRS_LAST_NAME $End/$CurrentUser{UserLastname}/gxms;
 
     # cleanup
-    $Param{Text} =~ s/$Tag.+?$End/-/gi;
     $Param{Text} =~ s/$Tag2.+?$End/-/gi;
 
     # ticket data
@@ -1327,64 +1300,102 @@ sub _Replace {
     # cleanup
     $Param{Text} =~ s/$Tag.+?$End/-/gi;
 
-    # get customer params and replace it with <OTRS_CUSTOMER_...
-    my %Data = %{ $Param{Data} };
+    # get customer and agent params and replace it with <OTRS_CUSTOMER_... or <OTRS_AGENT_...
+    my %ArticleData = (
+        'OTRS_CUSTOMER_' => $Param{Data}      || {},
+        'OTRS_AGENT_'    => $Param{DataAgent} || {},
+    );
 
-    # html quoting of content
-    if ( $Param{RichText} ) {
+    # use a list to get customer first
+    for my $DataType (qw(OTRS_CUSTOMER_ OTRS_AGENT_)) {
+        my %Data = %{ $ArticleData{$DataType} };
 
-        ATTRIBUTE:
-        for my $Attribute ( sort keys %Data ) {
-            next ATTRIBUTE if !$Data{$Attribute};
-            $Data{$Attribute} = $Kernel::OM->Get('Kernel::System::HTMLUtils')->ToHTML(
-                String => $Data{$Attribute},
-            );
-        }
-    }
-    if (%Data) {
+        # HTML quoting of content
+        if (
+            $Param{RichText}
+            && ( !$Data{ContentType} || $Data{ContentType} !~ /application\/json/ )
+            )
+        {
 
-        # check if original content isn't text/plain, don't use it
-        if ( $Data{'Content-Type'} && $Data{'Content-Type'} !~ /(text\/plain|\btext\b)/i ) {
-            $Data{Body} = '-> no quotable message <-';
+            ATTRIBUTE:
+            for my $Attribute ( sort keys %Data ) {
+                next ATTRIBUTE if !$Data{$Attribute};
+
+                $Data{$Attribute} = $Kernel::OM->Get('Kernel::System::HTMLUtils')->ToHTML(
+                    String => $Data{$Attribute},
+                );
+            }
         }
 
-        # replace <OTRS_CUSTOMER_*> tags
-        $Tag = $Start . 'OTRS_CUSTOMER_';
-        $HashGlobalReplace->( $Tag, %Data );
+        if (%Data) {
 
-        # replace <OTRS_CUSTOMER_BODY> and <OTRS_COMMENT> tags
-        for my $Key (qw(OTRS_CUSTOMER_BODY OTRS_COMMENT)) {
-            $Tag = $Start . $Key;
-            if ( $Param{Text} =~ /$Tag$End(\n|\r|)/g ) {
-                my $Line       = 2500;
-                my @Body       = split( /\n/, $Data{Body} );
+            # Check if content type is JSON
+            if ( $Data{'ContentType'} && $Data{'ContentType'} =~ /application\/json/ ) {
+
+                # if article is chat related
+                if ( $Data{'ArticleType'} =~ /chat/ ) {
+
+                    # remove spaces
+                    $Data{Body} =~ s/\n/ /gms;
+
+                    my $Body = $Kernel::OM->Get('Kernel::System::JSON')->Decode(
+                        Data => $Data{Body},
+                    );
+
+                    # replace body with HTML text
+                    $Data{Body} = $Kernel::OM->Get('Kernel::Output::HTML::Layout')->Output(
+                        TemplateFile => "ChatDisplay",
+                        Data         => {
+                            ChatMessages => $Body,
+                        },
+                    );
+                }
+            }
+
+            # check if original content isn't text/plain, don't use it
+            if ( $Data{'Content-Type'} && $Data{'Content-Type'} !~ /(text\/plain|\btext\b)/i ) {
+                $Data{Body} = '-> no quotable message <-';
+            }
+
+            # replace <OTRS_CUSTOMER_*> and <OTRS_AGENT_*> tags
+            $Tag = $Start . $DataType;
+            $HashGlobalReplace->( $Tag, %Data );
+
+            # prepare body (insert old email) <OTRS_CUSTOMER_EMAIL[n]>, <OTRS_CUSTOMER_NOTE[n]>
+            #   <OTRS_CUSTOMER_BODY[n]>, <OTRS_AGENT_EMAIL[n]>..., <OTRS_COMMENT>
+            if ( $Param{Text} =~ /$Start(?:(?:$DataType(EMAIL|NOTE|BODY)\[(.+?)\])|(?:OTRS_COMMENT))$End/g ) {
+
+                my $Line       = $2 || 2500;
                 my $NewOldBody = '';
-                COUNTER:
-                for ( my $i = 0; $i < $Line; $i++ ) {
-                    if ( $#Body >= $i ) {
+                my @Body       = split( /\n/, $Data{Body} );
+
+                for my $Counter ( 0 .. $Line - 1 ) {
+
+                    # 2002-06-14 patch of Pablo Ruiz Garcia
+                    # http://lists.otrs.org/pipermail/dev/2002-June/000012.html
+                    if ( $#Body >= $Counter ) {
 
                         # add no quote char, do it later by using DocumentCleanup()
                         if ( $Param{RichText} ) {
-                            $NewOldBody .= $Body[$i];
+                            $NewOldBody .= $Body[$Counter];
                         }
 
                         # add "> " as quote char
                         else {
-                            $NewOldBody .= "> $Body[$i]";
+                            $NewOldBody .= "> $Body[$Counter]";
                         }
 
                         # add new line
-                        if ( $i < ( $Line - 1 ) ) {
+                        if ( $Counter < ( $Line - 1 ) ) {
                             $NewOldBody .= "\n";
                         }
                     }
-                    else {
-                        last COUNTER;
-                    }
+                    $Counter++;
                 }
+
                 chomp $NewOldBody;
 
-                # html quoting of content
+                # HTML quoting of content
                 if ( $Param{RichText} && $NewOldBody ) {
 
                     # remove trailing new lines
@@ -1400,127 +1411,95 @@ sub _Replace {
                 }
 
                 # replace tag
-                $Param{Text} =~ s/$Tag$End/$NewOldBody/g;
+                $Param{Text}
+                    =~ s/$Start(?:(?:$DataType(EMAIL|NOTE|BODY)\[(.+?)\])|(?:OTRS_COMMENT))$End/$NewOldBody/g;
             }
-        }
 
-        # replace <OTRS_CUSTOMER_EMAIL[]> tags
-        $Tag = $Start . 'OTRS_CUSTOMER_EMAIL';
-        if ( $Param{Text} =~ /$Tag\[(.+?)\]$End/g ) {
-            my $Line       = $1;
-            my @Body       = split( /\n/, $Data{Body} );
-            my $NewOldBody = '';
-            for ( my $i = 0; $i < $Line; $i++ ) {
+            # replace <OTRS_CUSTOMER_SUBJECT[]>  and  <OTRS_AGENT_SUBJECT[]> tags
+            $Tag = "$Start$DataType" . 'SUBJECT';
+            if ( $Param{Text} =~ /$Tag\[(.+?)\]$End/g ) {
 
-                # 2002-06-14 patch of Pablo Ruiz Garcia
-                # http://lists.otrs.org/pipermail/dev/2002-June/000012.html
-                if ( $#Body >= $i ) {
-
-                    # add no quote char, do it later by using DocumentCleanup()
-                    if ( $Param{RichText} ) {
-                        $NewOldBody .= $Body[$i];
-                    }
-
-                    # add "> " as quote char
-                    else {
-                        $NewOldBody .= "> $Body[$i]";
-                    }
-
-                    # add new line
-                    if ( $i < ( $Line - 1 ) ) {
-                        $NewOldBody .= "\n";
-                    }
-                }
-            }
-            chomp $NewOldBody;
-
-            # html quoting of content
-            if ( $Param{RichText} && $NewOldBody ) {
-
-                # remove trailing new lines
-                for ( 1 .. 10 ) {
-                    $NewOldBody =~ s/(<br\/>)\s{0,20}$//gs;
-                }
-
-                # add quote
-                $NewOldBody = "<blockquote type=\"cite\">$NewOldBody</blockquote>";
-                $NewOldBody = $Kernel::OM->Get('Kernel::System::HTMLUtils')->DocumentCleanup(
-                    String => $NewOldBody,
+                my $SubjectChar = $1;
+                my $Subject     = $Kernel::OM->Get('Kernel::System::Ticket')->TicketSubjectClean(
+                    TicketNumber => $Ticket{TicketNumber},
+                    Subject      => $Data{Subject},
                 );
+
+                $Subject =~ s/^(.{$SubjectChar}).*$/$1 [...]/;
+                $Param{Text} =~ s/$Tag\[.+?\]$End/$Subject/g;
             }
 
-            # replace tag
-            $Param{Text} =~ s/$Tag\[.+?\]$End/$NewOldBody/g;
+            if ( $DataType eq 'OTRS_CUSTOMER_' ) {
+
+                # Arnold Ligtvoet - otrs@ligtvoet.org
+                # get <OTRS_EMAIL_DATE[]> from body and replace with received date
+                use POSIX qw(strftime);
+                $Tag = $Start . 'OTRS_EMAIL_DATE';
+
+                if ( $Param{Text} =~ /$Tag\[(.+?)\]$End/g ) {
+
+                    my $TimeZone = $1;
+                    my $EmailDate = strftime( '%A, %B %e, %Y at %T ', localtime );    ## no critic
+                    $EmailDate .= "($TimeZone)";
+                    $Param{Text} =~ s/$Tag\[.+?\]$End/$EmailDate/g;
+                }
+            }
         }
 
-        # replace <OTRS_CUSTOMER_SUBJECT[]> tags
-        $Tag = $Start . 'OTRS_CUSTOMER_SUBJECT';
-        if ( $Param{Text} =~ /$Tag\[(.+?)\]$End/g ) {
+        if ( $DataType eq 'OTRS_CUSTOMER_' ) {
 
-            my $SubjectChar = $1;
-            my $Subject     = $Kernel::OM->Get('Kernel::System::Ticket')->TicketSubjectClean(
-                TicketNumber => $Ticket{TicketNumber},
-                Subject      => $Data{Subject},
-            );
+            # get and prepare realname
+            $Tag = $Start . 'OTRS_CUSTOMER_REALNAME';
+            if ( $Param{Text} =~ /$Tag$End/i ) {
 
-            $Subject =~ s/^(.{$SubjectChar}).*$/$1 [...]/;
-            $Param{Text} =~ s/$Tag\[.+?\]$End/$Subject/g;
+                my $From = '';
+
+                if ( $Ticket{CustomerUserID} ) {
+
+                    $From = $Kernel::OM->Get('Kernel::System::CustomerUser')->CustomerName(
+                        UserLogin => $Ticket{CustomerUserID}
+                    );
+                }
+
+                # try to get the real name directly from the data
+                $From //= $Recipient{RealName};
+
+                # get real name based on reply-to
+                if ( $Data{ReplyTo} ) {
+                    $From = $Data{ReplyTo} || '';
+                }
+
+                # generate real name based on sender line
+                if ( !$From ) {
+                    $From = $Data{From} || '';
+
+                    # remove email addresses
+                    $From =~ s/&lt;.*&gt;|<.*>|\(.*\)|\"|&quot;|;|,//g;
+
+                    # remove leading/trailing spaces
+                    $From =~ s/^\s+//g;
+                    $From =~ s/\s+$//g;
+                }
+
+                # replace <OTRS_CUSTOMER_REALNAME> with from
+                $Param{Text} =~ s/$Tag$End/$From/g;
+            }
         }
-
-        # Arnold Ligtvoet - otrs@ligtvoet.org
-        # get <OTRS_EMAIL_DATE[]> from body and replace with received date
-        use POSIX qw(strftime);
-        $Tag = $Start . 'OTRS_EMAIL_DATE';
-
-        if ( $Param{Text} =~ /$Tag\[(.+?)\]$End/g ) {
-
-            my $TimeZone = $1;
-            my $EmailDate = strftime( '%A, %B %e, %Y at %T ', localtime );    ## no critic
-            $EmailDate .= "($TimeZone)";
-            $Param{Text} =~ s/$Tag\[.+?\]$End/$EmailDate/g;
-        }
-    }
-
-    # get and prepare realname
-    $Tag = $Start . 'OTRS_CUSTOMER_REALNAME';
-    if ( $Param{Text} =~ /$Tag$End/i ) {
-
-        my $From;
-
-        if ( $Ticket{CustomerUserID} ) {
-
-            $From = $Kernel::OM->Get('Kernel::System::CustomerUser')->CustomerName(
-                UserLogin => $Ticket{CustomerUserID}
-            );
-        }
-
-        # generate real name based on sender line
-        if ( !$From ) {
-            $From = $Data{To} || '';
-
-            # remove email addresses
-            $From =~ s/&lt;.*&gt;|<.*>|\(.*\)|\"|&quot;|;|,//g;
-
-            # remove leading/trailing spaces
-            $From =~ s/^\s+//g;
-            $From =~ s/\s+$//g;
-        }
-
-        # replace <OTRS_CUSTOMER_REALNAME> with from
-        $Param{Text} =~ s/$Tag$End/$From/g;
     }
 
     # get customer data and replace it with <OTRS_CUSTOMER_DATA_...
     $Tag  = $Start . 'OTRS_CUSTOMER_';
     $Tag2 = $Start . 'OTRS_CUSTOMER_DATA_';
 
-    if ( $Ticket{CustomerUserID} ) {
+    if ( $Ticket{CustomerUserID} || $Param{Data}->{CustomerUserID} ) {
+
+        my $CustomerUserID = $Param{Data}->{CustomerUserID} || $Ticket{CustomerUserID};
 
         my %CustomerUser = $Kernel::OM->Get('Kernel::System::CustomerUser')->CustomerUserDataGet(
-            User => $Ticket{CustomerUserID},
+            User => $CustomerUserID,
         );
 
-        # html quoting of content
+        # HTML quoting of content
         if ( $Param{RichText} ) {
 
             ATTRIBUTE:
@@ -1539,6 +1518,10 @@ sub _Replace {
     # cleanup all not needed <OTRS_CUSTOMER_DATA_ tags
     $Param{Text} =~ s/(?:$Tag|$Tag2).+?$End/-/gi;
 
+    # cleanup all not needed <OTRS_AGENT_ tags
+    $Tag = $Start . 'OTRS_AGENT_';
+    $Param{Text} =~ s/$Tag.+?$End/-/gi;
+
     return $Param{Text};
 }
 
